import { Request, Response, Router } from 'express';
import { AnimalRepository } from '../repository';
import { createAnimalScheme, updateAnimalScheme } from '../models/animalModels';
import { object, string, ValidationError } from 'yup';

const AnimalController = Router();

AnimalController.get('/', async (_: Request, res: Response) => {
    const animals = await AnimalRepository.getAll();

    res.send({
        items: animals,
        message: `Succesfully listed ${animals.length} animals`
    })
})

AnimalController.get('/:id', async (req: Request, res: Response) => {
    try {
        const { id } = await object({ id: string().required() }).validate(req.params);
        const animal = await AnimalRepository.getSingle(id);
        
        if (animal) {
            res.send({ item: animal, message: `Succesfully listed animal` });
        } else {
            res.status(404).send({ message: `Animal with id ${id} not found` });
        }
    } catch(e) {
        if (e instanceof ValidationError) {
            res.status(400).send({ message: e.errors });
        } else {
            res.status(500).send({ message: "Somethink went wrong"});
        }
    }
})


AnimalController.post('/', async (req: Request, res: Response) => {
    try {
        const data = await createAnimalScheme.validate(req.body);
        const animal = await AnimalRepository.createSingle(data);
        res.send({ item: animal, message: `Successfully created animal` })
    } catch(e) {
        if (e instanceof ValidationError) {
            res.status(400).send({ message: e.errors });
        } else {
            res.status(500).send({ message: "Somethink went wrong"});
        }
    }
})

AnimalController.put('/:id', async (req: Request, res: Response) => {
    try {
        const data = await updateAnimalScheme.validate(req.body);
        const { id } = await object({ id: string().required() }).validate(req.params);
        
        const animal = await AnimalRepository.updateSingle(id, data);
        res.send({ item: animal, message: `Successfully updated animal` });
    } catch(e) {
        if (e instanceof ValidationError) {
            res.status(400).send({ message: e.errors });
        } else {
            res.status(500).send({ message: "Somethink went wrong"});
        }
    }
})

AnimalController.delete('/:id', async (req: Request, res: Response) => {
    try {
        const { id } = await object({ id: string().required() }).validate(req.params);
        
        const animal = await AnimalRepository.deleteSingle(id);
        if (animal) {
            res.send({ item: animal, message: `Succesfully deleted animal with id ${animal.id}` });
        } else {
            res.status(404).send({ message: `Animal with id ${id} not found` });
        }
    } catch(e) {
        if (e instanceof ValidationError) {
            res.status(400).send({ message: e.errors });
        } else {
            res.status(500).send({ message: "Somethink went wrong"});
        }
    }
})


export default AnimalController;